#!/usr/bin/perl

######
# perlbot: EXPERIMENTAL IRC basic bot with scripting capabilities
# Author: Jimmy Scott
# Nick: Sick`
# Mail: jimmy *at* inet-solutions.be
# Address:      Jimmy Scott
#               Goordijk 66
#               2930 BRASSCHAAT
#               BELGIUM
# source: http://pub.devbox.be
###
#
# Copyright (C) 2003 Jimmy Scott <http://www.devbox.be>
#
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   1. Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#   3. The names of the authors may not be used to endorse or promote products
#      derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
######

use Socket;
use config::conf;
use botscript::on_nick;
use botscript::on_raw;
use botscript::on_join;
use botscript::on_action;
use botscript::on_ctcp;
use botscript::on_text;
use botscript::on_notice;
use botscript::on_mode;
use botscript::on_kick;

#
# CONNECTING
#

$version="v2.0-beta";
$proto = getprotobyname('tcp');
socket($socket, PF_INET, SOCK_STREAM, $proto);
$sin = sockaddr_in($server_port,inet_aton("$server")) || warn "Socket: $!\n";
connect($socket,$sin);
print "perlbot $version initializing...\n";

#
# ON SOCKET ACK
#

send($socket, "USER $ident .$modes. $ident :$realname\n", 0);
send($socket, "NICK $me\n", 0);

while (1)
{

  #
  # ON SOCKET RECEIVE
  #

  $read="";
  until ($read=~/(.+)\n/)
  {
    recv($socket,$bit,1,0);
    $read=$read . $bit;
  }

  #
  # ON SERVER PING
  #

  if ($read=~/^PING\s:(.+)/)
  {
    send($socket, "PONG $1\n", 0);
  }

  #
  # ON SERVER RAW
  #

  if ($read=~/^:(\S+)\s(\d\d\d)\s$me\s(.+)/)
  {
    local $srv_name=$1;
    local $srv_rawevent=$2;
    local $srv_message=$3;
    on_raw;
  }

  #
  # ON JOIN
  #

  if ($read=~/^:(\S+)\sJOIN\s:(\S+)/)
  {
    local $join=$1;
    local $channel=$2;
    $join=~/^(.+)!(.+)@(.+)$/;
    local $joinnick=$1;
    local $joinident=$2;
    local $joinhost=$3;
    on_join;
  }

  #
  # ON PRIVMSG
  #

  if ($read=~/^:(\S+)\sPRIVMSG\s(\S+)\s:(.+)/)
  {
    local $msgfrom=$1;
    local $msgto=$2;
    local $msgtext=$3;
    $msgfrom=~/^(.+)!(.+)@(.+)$/;
    local $msgnick=$1;
    local $msgident=$2;
    local $msghost=$3;

    #
    # ON EVENT
    #

    if ($msgtext=~/^(\S+)\s?(.+)?/)
    {
      local $event=$1;
      local $event_text=$2;

      if ($event eq ACTION)
      {
        on_action;
      }
      else
      {
        on_ctcp;
      }
    }
    else
    {
      on_text;
    }
  }

  #
  # On notice
  #

  if ($read=~/^:(\S+)\sNOTICE\s(\S+)\s:(.+)/)
  {
    local $msgfrom=$1;
    local $msgto=$2;
    local $msgtext=$3;
    $msgfrom=~/^(.+)!(.+)@(.+)$/;
    local $msgnick=$1;
    local $msgident=$2;
    local $msghost=$3;
    on_notice;
  }

  #
  # On Mode
  #

  if ($read=~/^:(\S+)\sMODE\s(\S+)\s(.+)\s(.+)./)
  {
    local $modefrom=$1;
    local $modechan=$2;
    local $modechange=$3;
    local $modeon=$4; chomp $modeon;
    $modefrom=~/^(.+)!(.+)@(.+)$/;
    local $modenick=$1;
    local $modeident=$2;
    local $modehost=$3;
    on_mode;
  }

  #
  # On kick
  #

  if ($read=~/^:(\S+)\sKICK\s(\S+)\s(\S+)\s:(.+)/)
  {
    local $kickfrom=$1;
    local $channel=$2;
    local $knick=$3;
    local $kickmsg=$4;
    $kickfrom=~/^(.+)!(.+)@(.+)$/;
    local $nick=$1;
    local $ident=$2;
    local $host=$3;
    on_kick;
  }

  #
  # On nick
  #

  if ($read=~/^:(\S+)\sNICK\s:(.+)/)
  {
    local $nickfrom=$1;
    local $newnick=$2;
    $nickfrom=~/^(.+)!(.+)@(.+)$/;
    local $nick=$1;
    local $ident=$2;
    local $host=$3;
    on_nick;
  }

  #
  # Debug line
  #
  
  # print "- $read\n";
}

